/// <reference path="reference/index.js" />
/// <reference path="reference/jquery.min.js" />

jQuery.noConflict();

(function ($, PLUGIN_ID) {
    "use strict";

    var root_url, client_id, client_secret, redirect_uri, service_id, service_password, username, password;

    var conf = kintone.plugin.app.getConfig(PLUGIN_ID);
    if (conf) {
        root_url = conf['root_url'];
        client_id = conf['client_id'];
        redirect_uri = conf['redirect_uri'];

        $('#root_url').val(root_url);
        $('#client_id').val(client_id);
        $('#redirect_uri').val(redirect_uri);
    }

    if (root_url) {
        // secret proxy config
        var proxyConfig = kintone.plugin.app.getProxyConfig(root_url + 'Auth/PreAuthorize', 'POST');
        if (proxyConfig) {
            service_id = proxyConfig.data['service_id'];
            service_password = proxyConfig.data['service_password'];
            username = proxyConfig.data['username'];
            password = proxyConfig.data['password'];

            $('#service_id').val(service_id);
            $('#service_password').val(service_password);
            $('#username').val(username);
            $('#password').val(password);
        }
        proxyConfig = kintone.plugin.app.getProxyConfig(root_url + 'Auth/Token', 'POST');
        if (proxyConfig) {
            var m = /^Basic (.+)/.exec(proxyConfig.headers['Authorization']);
            if (m) {
                var client_credentials = window.atob(m[1]);
                client_secret = client_credentials.split(':')[1];
            }
            $('#client_secret').val(client_secret);
        }
    }

    // 「保存する」クリック時に入力情報を設定する
    $('#plugin_submit').click(function () {
        root_url = $('#root_url').val();
        client_id = $('#client_id').val();
        client_secret = $('#client_secret').val();
        redirect_uri = $('#redirect_uri').val();
        service_id = $('#service_id').val();
        service_password = $('#service_password').val();
        username = $('#username').val();
        password = $('#password').val();

        if (!root_url || !client_id || !client_secret || !service_id || !service_password) {
            alert('必須項目が入力されていません。');
            return;
        }
        if (!utils.endsWith(root_url, '/')) {
            root_url = root_url + '/';
            $('#root_url').val(root_url);
        }
        if (!root_url.match(/^https?:\/\/.*\/$/i)) {
            alert('Web-API ルートURLの形式が不正です。');
            return;
        }

        // 設定保存
        // Auth/PreAuthorize (準備認可コード取得)
        var data = {
            'client_id': client_id,
            'service_id': service_id,
            'service_password': service_password,
            'username': username,
            'password': password
        };
        kintone.plugin.app.setProxyConfig(root_url + 'Auth/PreAuthorize', 'POST', {}, data, function () {
            // Auth/Token (アクセストークン取得)
            var client_credentials = window.btoa(client_id + ":" + client_secret);
            var header = {
                'Authorization': 'Basic ' + client_credentials
            }
            kintone.plugin.app.setProxyConfig(root_url + 'Auth/Token', 'POST', header, {}, function () {
                kintone.plugin.app.setProxyConfig(root_url + 'Auth/TokenInfo', 'POST', {}, {}, function () {
                    // public config
                    var data = {
                        'root_url': root_url,
                        'client_id': client_id,
                        'redirect_uri': redirect_uri
                    };
                    kintone.plugin.app.setConfig(data);
                })
            })
        })
    });

    // キャンセルボタン押下時の処理
    $('#plugin_cancel').click(function () {
        history.back();
    });

})(jQuery, kintone.$PLUGIN_ID);

var utils = {};

utils.startsWith = function (value, searchString, position) {
    position = position || 0;
    return value.lastIndexOf(searchString, position) === position;
}

utils.endsWith = function (value, searchString, position) {
    var subjectString = value.toString();
    if (position === undefined || position > subjectString.length) {
        position = subjectString.length;
    }
    position -= searchString.length;
    var lastIndex = subjectString.indexOf(searchString, position);
    return lastIndex !== -1 && lastIndex === position;
}


