/*
 * PCAクラウド Web-API プラグイン
 * Copyright (c) 2016 PCA CORPORATION
 *
 */

/// <reference path="../../reference/jquery.min.js" />
/// <reference path="../../reference/moment.min.js" />
/// <reference path="../../reference/es6-promise.min.js" />
/// <reference path="../../reference/index.js" />
/// <reference path="../../reference/show.js" />

jQuery.noConflict();

var pca = { webapi: {}, utils: {} };

jQuery(document).ready(function () {

    pca.PLUGIN_ID = 'ebmoffghamelgjmiebifmcpoelnlmobm';
    pca.LOCAL_STORAGE_PREFIX = pca.PLUGIN_ID + '_' + (/^\/(\w+)\/(\d+)/.test(location.pathname) ? RegExp.$1 + RegExp.$2 : "unknown");
    pca.LOCAL_STORAGE_ACCESS_TOKEN = pca.LOCAL_STORAGE_PREFIX + '_access_token';
    pca.LOCAL_STORAGE_REFRESH_TOKEN = pca.LOCAL_STORAGE_PREFIX + '_refresh_token';
    pca.LOCAL_STORAGE_EXPIRES_IN = pca.LOCAL_STORAGE_PREFIX + '_expires_in';
    pca.LOCAL_STORAGE_DATAAREA_NAME = pca.LOCAL_STORAGE_PREFIX + '_dataarea_name';

    var config = kintone.plugin.app.getConfig(pca.PLUGIN_ID);
    if (!config) {
        throw new Error('プラグインが設定されていません。');
    }

    pca.ROOT_URL = config['root_url'];
    pca.CLIENT_ID = config['client_id'];
    pca.REDIRECT_URI = config['redirect_uri'];
    pca.MIN_EXPIRES_IN = 30;

    if (!pca.utils.endsWith(pca.ROOT_URL, '/')) {
        pca.ROOT_URL = pca.ROOT_URL + '/';
    }
})

pca.webapi = {
    /// <summary>
    /// アクセストークン
    /// </summary>
    get access_token() { return localStorage.getItem(pca.LOCAL_STORAGE_ACCESS_TOKEN); },
    set access_token(v) { pca.utils.setLocalStorageItem(pca.LOCAL_STORAGE_ACCESS_TOKEN, v); },

    /// <summary>
    /// リフレッシュトークン
    /// </summary>
    get refresh_token() { return localStorage.getItem(pca.LOCAL_STORAGE_REFRESH_TOKEN); },
    set refresh_token(v) { pca.utils.setLocalStorageItem(pca.LOCAL_STORAGE_REFRESH_TOKEN, v); },

    /// <summary>
    /// アクセストークンの有効期限(Unix時間)
    /// </summary>
    get expires_in() { return Number(localStorage.getItem(pca.LOCAL_STORAGE_EXPIRES_IN)); },
    set expires_in(v) { pca.utils.setLocalStorageItem(pca.LOCAL_STORAGE_EXPIRES_IN, v); },

    /// <summary>
    /// 使用中のデータ領域名
    /// </summary>
    get dataarea_name() { return localStorage.getItem(pca.LOCAL_STORAGE_DATAAREA_NAME); },
    set dataarea_name(v) { pca.utils.setLocalStorageItem(pca.LOCAL_STORAGE_DATAAREA_NAME, v); },
}

pca.webapi.hasAccess = function () {
    /// <summary>
    /// 保持するアクセストークンが有効かを取得します。
    /// </summary>
    /// <return type="Boolean">アクセストークンが有効な場合は true, それ以外は false.</return>
    var self = this;

    if (self.access_token && self.expires_in - pca.utils.unixNow() >= pca.MIN_EXPIRES_IN) {
        return true;
    }
    if (self.refresh_token){
        return true;
    }
    return false;
}

pca.webapi.createHeaders = function (additional) {
    /// <summary>
    /// Authorization 他をセットしたヘッダーオブジェクトを作成します。
    /// additional に Content-Type, Accept がない場合はそれぞれに application/json をセットします。
    /// </summary>
    /// <param name="additional" type="Object">追加するヘッダーオブジェクト.</param>
    /// <returns type="Object"></returns>
    var self = this;

    var headers = {};
    if (this.access_token) {
        headers['Authorization'] = 'Bearer ' + self.access_token
    }

    var contans = function(obj, target){
        try{ return target in obj; }
        catch(e){ return false; }
    }

    if (!contans(additional, 'Content-Type')) {
        headers['Content-Type'] = 'application/json';
    }
    if (!contans(additional, 'Accept')) {
        headers['Accept'] = 'application/json';
    }
    if (additional) {
        jQuery.extend(headers, additional);
    }
    return headers;
}

pca.webapi.login = function (dataAreaName, callback, error) {
    /// <summary>
    /// OAuth 2.0 認可コードグラントで PCAクラウド Web-API のアクセストークンを取得します。
    /// </summary>
    /// <param name='dataAreaName' type='String'>ログイン後に選択するデータ領域名 (例: 'P10V01C001KON0001'). 無指定の場合は領域を選択しません。別途 pca.webapi.selectDataArea() を実行してください。</param>
    /// <param name='callback' type='function()'>(省略可) リクエストが完了したあとに実行する関数を指定します。</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;

    var popup = window.open('about:blank', '_blank');

    var promise = kintone.Promise.resolve()
    .then(function () {
        return kintone.plugin.app.proxy(pca.PLUGIN_ID, pca.ROOT_URL + 'Auth/PreAuthorize', 'POST', { 'Content-Type': 'application/json' }, {})
        .then(function (args) { // [b, status, h]
            var res = JSON.parse(args[0]);
            if (!pca.utils.isok(args[1])) {
                return kintone.Promise.reject(res.error);
            }
            var pre_authorization_code = res['pre_authorization_code'];

            var url = pca.ROOT_URL + 'Auth/Authorize?' + pca.utils.createFormData({
                'response_type': 'code',
                'scope': 'read',
                'client_id': pca.CLIENT_ID,
                'redirect_uri': pca.REDIRECT_URI,
                'pre_authorization_code': pre_authorization_code
            });
            popup.location = url;
            if (window.focus) {
                popup.focus();
            }

            var promisePoll = new kintone.Promise(function (resolve, reject) {
                // popup をポーリングして認可コードを取得
                var polling = setInterval(function () {
                    if (!popup || popup.closed || popup.closed === undefined) {
                        clearInterval(polling);
                        reject('認証がキャンセルされました。');
                        return;
                    }
                    try {
                        var loc = popup.location;
                        if (pca.utils.startsWith(loc.toString(), pca.REDIRECT_URI, 0)) {

                            var search = pca.utils.parseFormString(loc.search.substr(1));
                            if (search.code) {
                                resolve(search.code);
                            } else {
                                reject('認可コードが取得できません。' + search.error);
                            }
                            clearInterval(polling);
                            popup.close();
                        }
                    }
                    catch (err) {
                        // DOMException: Blocked a frame with origin "(current origin)" from accessing a cross-origin frame."
                    }
                }, 20);
            })

            return promisePoll;
        })
        .then(function (code) {
            // code: 認可コード
            var data = {
                'grant_type': 'authorization_code',
                'code': code,
                'redirect_uri': pca.REDIRECT_URI
            };
            return kintone.plugin.app.proxy(pca.PLUGIN_ID, pca.ROOT_URL + 'Auth/Token', 'POST', { 'Content-Type': 'application/x-www-form-urlencoded' }, pca.utils.createFormData(data));
        })
        .then(function (args) { // [b, status, h]
            var res = JSON.parse(args[0]);
            if (!pca.utils.isok(args[1])) {
                return kintone.Promise.reject('アクセストークン リクエストエラー。' + res.error);
            }
            self.access_token = res['access_token'];
            self.refresh_token = res['refresh_token'];
            self.expires_in = Number(res['expires_in'] + pca.utils.unixNow());
            self.dataarea_name = null;

            if (dataAreaName) {
                // データ領域選択
                return self.selectDataArea(dataAreaName);
            }
            return kintone.Promise.resolve();
        })
        .catch(function (reason) {
            popup.close();
            return kintone.Promise.reject(reason);
        })
    });

    promise.then(function () {
        if (callback) callback();
    }).catch(function (reason) {
        if (error) error(reason);
    });

    if (!callback) {
        return promise;
    }
}

pca.webapi.refreshToken = function (callback, error) {
    /// <summary>
    /// アクセストークンが期限切れの場合にトークンのリフレッシュを実行します。
    /// </summary>
    /// <param name='callback' type='function()'>(省略可) リクエストが完了したあとに実行する関数を指定します。</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <return type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</return>
    var self = this;

    var promise = kintone.Promise.resolve()
    .then(function () {
        if (self.access_token && self.expires_in - pca.utils.unixNow() >= pca.MIN_EXPIRES_IN) {
            return kintone.Promise.resolve();
        }
        if (!self.refresh_token) {
            return kintone.Promise.reject('ログインしてから実行してください。');
        }

        // トークンのリフレッシュ
        var data = {
            'grant_type': 'refresh_token',
            'refresh_token': self.refresh_token,
        }
        return kintone.plugin.app.proxy(pca.PLUGIN_ID, pca.ROOT_URL + 'Auth/Token', 'POST', { 'Content-Type': 'x-www-form-urlencoded' }, pca.utils.createFormData(data))
        .then(function (args) { // [b, status, h]
            var res = JSON.parse(args[0]);
            if (!pca.utils.isok(args[1])) {
                return kintone.Promise.reject('トークンのリフレッシュ時にエラーが発生しました。' + res.error);
            }
            self.access_token = res['access_token'];
            self.refresh_token = res['refresh_token'];
            self.expires_in = Number(res['expires_in'] + pca.utils.unixNow());

            if (self.dataarea_name) {
                // 指定されたデータ領域を選択
                return self.selectDataArea(self.dataarea_name)
                .catch(function (reason) {
                    self.dataarea_name = null;
                    return kintone.Promise.reject(reason);
                });
            }
            return kintone.Promise.resolve();
        })
        .catch(function (reason) {
            return kintone.Promise.reject(reason);
        });
    });

    promise.then(function () {
        if (callback) callback();
    }).catch(function (reason) {
        if (error) error(reason);
    })

    if (!callback) {
        return promise;
    }
}

pca.webapi.logout = function (callback, error) {
    /// <summary>
    /// PCAクラウド Web-API からログアウトします。
    /// </summary>
    /// <param name='callback' type='function()'>(省略可) リクエストが完了したあとに実行する関数を指定します。</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <return type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</return>
    var self = this;

    var promise = kintone.plugin.app.proxy(pca.PLUGIN_ID, pca.ROOT_URL + 'Logout', 'POST', self.createHeaders(), {})
    .then(function (args) { // [b, status, h]
        self.access_token = null;
        self.refresh_token = null;
        self.expires_in = null;
        self.dataarea_name = null;

        return kintone.Promise.resolve();
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function () {
        if (callback) callback();
    }).catch(function (reason) {
        if (error) error(reason);
    })

    if (!callback) {
        return promise;
    }
}

pca.webapi.getLogOnUser = function (callback, error) {
    /// <summary>
    /// ログイン中のユーザー情報を取得します。
    /// </summary>
    /// <param name='callback' type='function(Object result)'>(省略可) リクエストが完了したあとに実行する関数を指定します。result は BECommonSystemUser</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;

    var promise = self.refreshToken()
    .then(function () {
        return kintone.proxy(pca.ROOT_URL + 'GetLogOnUser', 'GET', self.createHeaders(), {})
        .then(function (args) { // [b, status, h]
            var res = JSON.parse(args[0]);
            if (!pca.utils.isok(args[1])) {
                return kintone.Promise.reject(res.Message);
            }
            var result = res.BECommonSystemUser;
            return kintone.Promise.resolve(result);
        })
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function (result) {
        if (callback) callback(result);
    }).catch(function (reason) {
        if (error) error(reason);
    });

    if (!callback) {
        return promise;
    }
}

pca.webapi.getLogOnDataArea = function (callback, error) {
    /// <summary>
    /// ログイン中のデータ領域情報を取得します。
    /// </summary>
    /// <param name='callback' type='function(Object result)'>(省略可) リクエストが完了したあとに実行する関数を指定します。result は BECommonDataArea</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;

    var promise = self.refreshToken()
    .then(function () {
        return kintone.proxy(pca.ROOT_URL + 'GetLogOnDataArea', 'GET', self.createHeaders(), {})
    })
    .then(function (args) { // [b, status, h]
        var res = JSON.parse(args[0]);
        if (!pca.utils.isok(args[1])) {
            return kintone.Promise.reject(res.Message);
        }
        var result = res.BECommonDataArea;
        return kintone.Promise.resolve(result);
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function (result) {
        if (callback) callback(result);
    }).catch(function (reason) {
        if (error) error(reason);
    });

    if (!callback) {
        return promise;
    }
}

pca.webapi.findDataArea = function (callback, error) {
    /// <summary>
    /// ログイン中のユーザーが使用可能なデータ領域を取得します。
    /// </summary>
    /// <param name='callback' type='function(Object result)'>(省略可) リクエストが完了したあとに実行する関数を指定します。result は BECommonDataArea の配列</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;

    var promise = self.refreshToken()
    .then(function () {
        return kintone.proxy(pca.ROOT_URL + 'FindDataArea', 'GET', self.createHeaders(), {})
        .then(function (args) { // [b, status, h]
            var res = JSON.parse(args[0]);
            if (!pca.utils.isok(args[1])) {
                return kintone.Promise.reject(res.Message);
            }
            var result = res.ArrayOfBECommonDataArea.BECommonDataArea;
            return kintone.Promise.resolve(result);
        })
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function (result) {
        if (callback) callback(result);
    }).catch(function (reason) {
        if (error) error(reason);
    });

    if (!callback) {
        return promise;
    }
}

pca.webapi.selectDataArea = function (dataAreaName, callback, error) {
    /// <summary>
    /// 使用するデータ領域を変更します。
    /// </summary>
    /// <param name="dataAreaName" type="String">選択するデータ領域名(例: 'P10V01C001KON0001')</param>
    /// <param name='callback' type='function()'>(省略可) リクエストが完了したあとに実行する関数を指定します。</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;
    self.dataarea_name = null;

    var promise = self.refreshToken()
    .then(function () {
        return kintone.proxy(pca.ROOT_URL + 'SelectDataArea', 'POST', self.createHeaders(), { 'DataArea': dataAreaName })
        .then(function (args) { // [b, status, h]
            if (!pca.utils.isok(args[1])) {
                var res = JSON.parse(args[0]);
                return kintone.Promise.reject(res.Message);
            }
            self.dataarea_name = dataAreaName;
            kintone.Promise.resolve();
        })
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function () {
        if (callback) callback();
    }).catch(function (reason) {
        if (error) error(reason);
    })

    if (!callback) {
        return promise;
    }
}

pca.webapi.send = function (uri, method, headers, data, callback, error) {
    /// <summary>
    /// Web-APIリクエストを送信します。
    /// </summary>
    /// <param name="uri" type="String">実行する Web-API のURI (例: "/Find/MasterSms?Code=0001")</param>
    /// <param name="method" type="String">HTTPメソッド。GET, POSTのいずれかを指定</param>
    /// <param name="headers" type="Object">リクエストヘッダを指定します。何も指定しない場合は {} を指定します。
    /// Content-Type, Accept を指定しない場合の規定値は 'application/json' になります。
    /// (例) { 'Content-Type': 'application/x-www-form-urlencoded' }</param>
    /// <param name="data" type="Object または String">リクエストに載せるデータ。 何も指定しない場合は{}を指定します。</param>
    /// <param name='callback' type='function([String responseText, Number httpStatusCode, Object responseHeaders])'>(省略可) リクエストが完了したあとに実行する関数を指定します。</param>
    /// <param name='error' type='function(String error)'>(省略可) リクエストが失敗した場合に実行する関数を指定します。</param>
    /// <returns type="kintone.Promise">引数のcallbackを省略した時、kintone.Promiseオブジェクトを返します。</returns>
    var self = this;

    if (pca.utils.startsWith(uri, '/')) {
        uri = uri.replace(/^\/+/, '');  // trim start '/'
    }
    data = data || {};

    var promise = self.refreshToken()
    .then(function () {
        return kintone.proxy(pca.ROOT_URL + uri, method, self.createHeaders(headers), data)
        .then(function (args) { // [b, status, h]
            return kintone.Promise.resolve(args);
        })
    })
    .catch(function (reason) {
        return kintone.Promise.reject(reason);
    });

    promise.then(function (args) {
        if (callback) callback(args);
    }).catch(function (reason) {
        if (error) error(reason);
    })

    if (!callback) {
        return promise;
    }
}

pca.utils.parseFormString = function (text) {
    /// <summary>
    /// フォーム文字列 ("key1=value1&key2=value2") を連想配列として取得します。
    /// </summary>
    /// <param name="text" type="String"></param>
    /// <returns type=""></returns>
    /// <summary>
    /// フォーム文字列 ("key1=value1&key2=value2") を連想配列として取得します。
    /// </summary>
    /// <param name="text" type="String"></param>
    /// <returns type=""></returns>
    var result = {};
    if (typeof (text) != "string") {
        return result;
    }
    var parameters = text.split('&');
    for (var i = 0; i < parameters.length; i++) {
        var element = parameters[i].split('=');
        var name = decodeURIComponent(element[0]);
        var value = decodeURIComponent(element[1]);
        result[name] = value;
    }
    return result;
}

pca.utils.createFormData = function (data) {
    /// <summary>
    /// 連想配列をフォーム文字列に変換します。
    /// </summary>
    /// <param name="data" type="type"></param>
    /// <returns type="String">x-www-form-urlencoded 文字列</returns>
    var array = [];
    for (var key in data) {
        var val = data[key];
        if (jQuery.isArray(val)) {
            for (var i = 0; i < val.length; i++) {
                array.push(key + '[]=' + encodeURIComponent(val[i]));
            }
        } else {
            array.push(key + '=' + encodeURIComponent(val));
        }
    }
    return array.join('&');
}

pca.utils.isok = function (status) {
    /// <summary>
    /// HTTPステータスコードの成功を判定します。
    /// </summary>
    /// <param name="status" type="String">HTTPステータスコード</param>
    /// <returns type="Boolean">ステータスコードが 200 系の場合は true, それ以外は false</returns>
    var n = Number(status);
    return (200 <= n && n < 300);
}

pca.utils.unixNow = function () {
    /// <summary>
    /// UNIX時間での現在時刻を取得します。
    /// </summary>
    return Math.floor(new Date().getTime() / 1000);
}

pca.utils.setLocalStorageItem = function (key, value) {
    /// <summary>
    /// localStorage に値をセットします。value が null 等の場合はキーを削除します。
    /// </summary>
    /// <param name="key" type="String">キー</param>
    /// <param name="value" type="String">値</param>
    if (!key)
        return;

    if (value) {
        localStorage.setItem(key, value);
    } else {
        localStorage.removeItem(key);
    }
}

pca.utils.startsWith = function (value, searchString, position) {
    position = position || 0;
    return value.lastIndexOf(searchString, position) === position;
}

pca.utils.endsWith = function (value, searchString, position) {
    var subjectString = value.toString();
    if (position === undefined || position > subjectString.length) {
        position = subjectString.length;
    }
    position -= searchString.length;
    var lastIndex = subjectString.indexOf(searchString, position);
    return lastIndex !== -1 && lastIndex === position;
}

pca.utils.getcache = function (key, data) {
    /// <summary>
    /// 現在の kintoneアプリのキャッシュデータを取得します。データの型は保存されます。
    /// localStorage に保存され、kintone アプリID別に管理されます。
    /// </summary>
    /// <param name="key" type="string">キー文字列</param>
    /// <param name="value" >値</param>
    /// <return>キャッシュデータ。見つからない場合は null</return>
    var key2 = pca.LOCAL_STORAGE_PREFIX + "_cache_" + key;

    try {
        var value = JSON.parse(localStorage.getItem(key2));
        return value.data;
    }
    catch (e) {
        return null;
    }
}

pca.utils.setcache = function (key, data) {
    /// <summary>
    /// 現在の kintoneアプリのキャッシュデータをセットします。データの型は保存されます。
    /// localStorage に保存され、kintone アプリID別に管理されます。
    /// </summary>
    /// <param name="key" type="string">キー文字列</param>
    /// <param name="value">値。null または undefined の場合はキーを削除します。</param>
    var key2 = pca.LOCAL_STORAGE_PREFIX + "_cache_" + key;

    if (data == null || data == undefined) {
        localStorage.removeItem(key2);
    } else {
        localStorage.setItem(key2, JSON.stringify({ "data" : data }));
    }
}

pca.utils.clearcache = function (keys) {
    /// <summary>
    /// 現在の kintoneアプリのキャッシュデータをクリアします。
    /// </summary>
    /// <param name="key" type="string[]">キャッシュをクリアするキーの配列。null または undefined の場合は全てのキーを対象とします。</param>
    var key_prefix = pca.LOCAL_STORAGE_PREFIX + "_cache_";

    if (!jQuery.isArray(keys) && keys !== null && keys !== undefined)
        return;

    Object.keys(localStorage).forEach(function (key) {
        var ismatch = false;
        if (jQuery.isArray(keys)) {
            ismatch = keys.filter(function (e) { return (key_prefix + e) === key }).length > 0;
        } else {
            ismatch = pca.utils.startsWith(key, key_prefix, 0);
        }
        if (ismatch) {
            localStorage.removeItem(key);
        }
    });
}

pca.utils.toDictionary = function (items, key, value) {
    /// <summary>
    /// 連想配列から辞書を作成します。
    /// 例: pca.utils.toDictionary (items, function (item) { return item.Code });
    /// </summary>
    /// <param name="items" type="Object">オブジェクト</param>
    /// <param name="key" type="function (item)">items の要素からキーを取得するセレクタ function</param>
    /// <param name="value" type="function (item)">items の要素からキーにマッチする値を取得するセレクタ function. 未指定の場合は item が値になります。</param>
    var dictionary = {};
    if (items) {
        for (var i = 0; i < items.length; i++) {
            var item = items[i];
            dictionary[key(item)] = value ? value(item) : item;
        }
    }
    return dictionary;
}


